/*
 *	RtTrend widget
 *
 *	Actually, two versions of the widget are defined here,
 *	and the DRAW_POLY macro can be defined to choose one of them.
 */

#ifdef _PHSLIB
	#include "/src/photon/lib/imports.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>

#if defined( INCLUDE_LOCAL )
	#include "RtTrend.h"
#else
	#include <photon/realtime/RtTrend.h>
#endif

#if ! defined( DRAW_POLY )
	/*	DRAW_POLY can be 1
	 *	if PgDrawPolygon() should be used instead of PgDrawTrend().
	 *	Note that trend->lines[i].data will then point
	 *	to an array of PhPoint_t structures, not shorts.
	 */
	#define DRAW_POLY 0
#endif


/*	Values for trend->modify */
#define START	3
#define DONE	2
#define DAMAGED	1
#define NEWDATA	0

static void change( RtTrendWidget_t *trend, RtTrendLines_t *line, unsigned nlines,
		short const *newdata, unsigned last_sample, unsigned nsamples
		) {
	PtWidget_t *widget = (PtWidget_t*) trend;
	PhRect_t rect; long scale;
	int toomuch, minval = INT_MAX, maxval = INT_MIN;
	unsigned first_sample;
	if	(	last_sample >= trend->n_data
		||	( scale = trend->max - trend->min ) <= 0
			)
		return;

	if	(	(	toomuch
			=	( first_sample = last_sample + nsamples )
			-	trend->n_data
				)
		>	0
			) {
		nsamples -= toomuch;
		first_sample = trend->n_data;
		}
	else
		toomuch = 0;

	trend->sample_size = 0;
	trend->modify = DAMAGED;

	PtHold();

	do	{
		newdata += toomuch;
		if ( last_sample <= line->valid_size ) {
			if ( line->valid_size < first_sample )
				line->valid_size = first_sample;
			#if DRAW_POLY
			{	PhPoint_t *dst = (PhPoint_t*)line->data + first_sample;
				short const *src = newdata;
				int i = nsamples;
				if ( trend->flags & Rt_TREND_VERTICAL )
					do	{
						int val;
						if ( ( val = (--dst)->x ) > maxval )
							maxval = val;
						else
							if ( val < minval )
								minval = val;
						dst->x = val
						=	trend->width * ( (long)trend->max - *src++ )
						/	scale;
						if ( val > maxval )
							maxval = val;
						else
							if ( val < minval )
								minval = val;
						}	while ( --i );
				else
					do	{
						int val;
						if ( ( val = (--dst)->y ) > maxval )
							maxval = val;
						else
							if ( val < minval )
								minval = val;
						dst->y = val
						=	trend->width * ( (long)trend->max - *src++ )
						/	scale;
						if ( val > maxval )
							maxval = val;
						else
							if ( val < minval )
								minval = val;
						}	while ( --i );
				}
			#else /* ! DRAW_POLY */
			{	short *dst = line->data + first_sample;
				short const *src = newdata;
				int i = nsamples;
				do	{
					int val;
					if ( ( val = *--dst ) > maxval )
						maxval = val;
					else
						if ( val < minval )
							minval = val;
					*dst = val
					=	trend->width * ( (long)trend->max - *src++ )
					/	scale;
					if ( val > maxval )
						maxval = val;
					else
						if ( val < minval )
							minval = val;
					}	while ( --i );
				}
			#endif /* DRAW_POLY */
			}
		newdata += nsamples;
		}	while ( --nlines );

	/* Rescale last_sample and nsamples to pixels */
	if ( last_sample ) {
		last_sample = ( last_sample - 1 ) * trend->inc;
		nsamples = ( nsamples + 1 ) * trend->inc;
		}
	else
		nsamples *= trend->inc;
	PtBasicInteriorExtent( widget, &rect );
	if ( trend->flags & Rt_TREND_HORIZONTAL ) {
		if ( trend->flags & Rt_TREND_RIGHT_TO_LEFT )
			rect.ul.x = ( rect.lr.x -= last_sample ) - nsamples;
		else
			rect.lr.x = ( rect.ul.x += last_sample ) + nsamples;
		if ( ( maxval += rect.ul.y ) < rect.lr.y )
			rect.lr.y = maxval;
		if ( minval > 0 )
			rect.ul.y += minval;
		}
	else {
		if ( trend->flags & Rt_TREND_BOTTOM_TO_TOP )
			rect.ul.y = ( rect.lr.y -= last_sample ) - nsamples;
		else
			rect.lr.y = ( rect.ul.y += last_sample ) + nsamples;
		if ( ( maxval += rect.ul.x ) < rect.lr.x )
			rect.lr.x = maxval;
		if ( minval > 0 )
			rect.ul.x += minval;
		}
	PtDamageExtent( widget, &rect );
	PtRelease();
	}


void RtTrendChangeData( PtWidget_t *widget,
		short const *newdata, unsigned last_sample, unsigned nsamples
		) {
	RtTrendWidget_t *trend = (RtTrendWidget_t*) widget;
	if ( trend->trend_count != 0 )
		change( trend, trend->lines, trend->trend_count, newdata, last_sample, nsamples );
	}

void RtTrendChangeTrendData( PtWidget_t *widget, unsigned tn,
		short const *newdata, unsigned last_sample, unsigned nsamples
		) {
	RtTrendWidget_t *trend = (RtTrendWidget_t*) widget;
	if ( tn < trend->trend_count )
		change( trend, &trend->lines[tn], 1, newdata, last_sample, nsamples );
	}

